#pragma once

constexpr UINT WM_TRAYNOTIFY = WM_USER + 100;

#include "..\NTray.h"
#include "resource.h"
#include "AboutDlg.h"


class CMainFrame : public CFrameWindowImpl<CMainFrame>,
                   public CUpdateUI<CMainFrame>,
                   public CMessageFilter,
                   public CIdleHandler
{
public:
//Constructors / Destructors
#pragma warning(suppress: 26455)
  CMainFrame() : m_bHappy{true},
                 m_nTimerID{0},
                 m_nTimerCount{0},
                 m_pCurrentAboutDlg{nullptr}
  {
#pragma warning(suppress: 26446)
    m_Icons[0] = CTrayNotifyIcon::LoadIcon(IDR_HAPPY);
#pragma warning(suppress: 26446)
    m_Icons[1] = CTrayNotifyIcon::LoadIcon(IDR_SAD);
  }
  CMainFrame(const CMainFrame&) = delete;
  CMainFrame(CMainFrame&&) = delete;
  ~CMainFrame() = default; //NOLINT(modernize-use-override)

//Methods
  CMainFrame& operator=(const CMainFrame&) = delete;
  CMainFrame& operator=(CMainFrame&&) = delete;

//Member variables
  CCommandBarCtrl m_CmdBar;
  std::array<HICON, 2> m_Icons;
  bool m_bHappy;
  CDC m_TrayIconDC;
  CBitmap m_BitmapTrayIcon;
  UINT_PTR m_nTimerID;
  int m_nTimerCount;
  CTrayNotifyIcon m_TrayIcon1;
  CTrayNotifyIcon m_TrayIcon2;
  CTrayNotifyIcon m_TrayIcon3;
  CAboutDlg* m_pCurrentAboutDlg;

  BOOL PreTranslateMessage(MSG* pMsg) override
  {
    return CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg);
  }

  BOOL OnIdle() noexcept override
  {
    return FALSE;
  }

#pragma warning(suppress: 26440 26477)
  DECLARE_FRAME_WND_CLASS(nullptr, IDR_MAINFRAME)

#pragma warning(suppress: 26440)
  BEGIN_UPDATE_UI_MAP(CMainFrame) //NOLINT(modernize-avoid-c-arrays)
    UPDATE_ELEMENT(ID_HAPPY, UPDUI_MENUPOPUP)
    UPDATE_ELEMENT(ID_SAD, UPDUI_MENUPOPUP)
    UPDATE_ELEMENT(ID_SHOW, UPDUI_MENUPOPUP)
#pragma warning(suppress: 26485)
  END_UPDATE_UI_MAP()

#pragma warning(suppress: 26433 26496)
  BEGIN_MSG_MAP(CMainFrame)
    MSG_WM_CREATE(OnCreate)
    MSG_WM_CLOSE(OnClose)
    MSG_WM_TIMER(OnTimer)
    MESSAGE_HANDLER_EX(WM_TRAYNOTIFY, OnTrayNotification)
    COMMAND_ID_HANDLER_EX(ID_APP_EXIT, OnFileExit)
    COMMAND_ID_HANDLER_EX(ID_HAPPY, OnHappy)
    COMMAND_ID_HANDLER_EX(ID_HIDE, OnHide)
    COMMAND_ID_HANDLER_EX(ID_SHOW, OnShow)
    COMMAND_ID_HANDLER_EX(ID_SAD, OnSad)
    COMMAND_ID_HANDLER_EX(ID_ANIMATED, OnAnimated)
    COMMAND_ID_HANDLER_EX(ID_APP_ABOUT, OnAppAbout)
    CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
    CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
#pragma warning(suppress: 26477)
  END_MSG_MAP()

  LRESULT OnCreate(LPCREATESTRUCT /*lpCS*/)
  {
    //create command bar window
    HWND hWndCmdBar{m_CmdBar.Create(m_hWnd, rcDefault, nullptr, ATL_SIMPLE_CMDBAR_PANE_STYLE)};

    //attach menu
    m_CmdBar.AttachMenu(GetMenu());

    //load command bar images
    m_CmdBar.LoadImages(IDR_MAINFRAME);

    CreateSimpleReBar(ATL_SIMPLE_REBAR_NOBORDER_STYLE);
    AddSimpleReBarBand(hWndCmdBar);

    UISetCheck(ID_HAPPY, m_bHappy);
    UISetCheck(ID_SAD, !m_bHappy);
    UIEnable(ID_SHOW, TRUE); //Since we are creating the first tray icon initially hidden

    //register object for message filtering and idle updates
    CMessageLoop* pLoop{_Module.GetMessageLoop()};
#pragma warning(suppress: 26477)
    ATLASSUME(pLoop != nullptr);
    pLoop->AddMessageFilter(this);
    pLoop->AddIdleHandler(this);

#pragma warning(suppress: 26446)
    if (!m_TrayIcon1.Create(this, IDR_TRAYPOPUP, _T("Static Tray Icon"), _T("Demo Tray Application"), _T("Static Tray Icon"), 10, CTrayNotifyIcon::BalloonStyle::User, m_Icons[0], WM_TRAYNOTIFY, 0, true, false, false, nullptr, false, false)) //For testing purposes initally creating a tray icon hidden!
    //if (!m_TrayIcon1.Create(this, IDR_TRAYPOPUP, _T("First Tray Icon"), m_hIcons[0], WM_TRAYNOTIFY)) //Uncomment this line and comment out line above if you want to start with a tray icon without animations
    {
      ATLTRACE(_T("Failed to create tray icon 1\n"));
      return -1;
    }
    //For testing purposes, change the default menu item on the first icon to be the bottom item on the context menu
    m_TrayIcon1.SetDefaultMenuItem(ID_APP_ABOUT, false);

    if (!m_TrayIcon2.Create(this, IDR_TRAYPOPUP2, _T("Animated Tray Icon"), _T("Use the tray context menu to unhide the \"first tray icon\"!"), _T("Animated Tray Icon"), 10, CTrayNotifyIcon::BalloonStyle::Info, m_Icons.data(), 2, 600, WM_TRAYNOTIFY, IDR_TRAYPOPUP2))
    //if (!m_TrayIcon2.Create(this, IDR_TRAYPOPUP2, _T("Second Tray Icon"), _T("Demo Tray Application"), _T("Second Tray Icon"), 10, CTrayNotifyIcon::Info, m_Icons[0], WM_TRAYNOTIFY, IDR_TRAYPOPUP2)) //Uncomment this line and comment out line above if you want a tray icon without animations
    {
      ATLTRACE(_T("Failed to create tray icon 2\n"));
      return -1;
    }

    //Create the dynamic systray
    if (!CTrayNotifyIcon::GetDynamicDCAndBitmap(&m_TrayIconDC, &m_BitmapTrayIcon))
    {
      ATLTRACE(_T("Failed to create tray icon 3\n"));
      return -1;
    }
    if (!m_TrayIcon3.Create(this, IDR_TRAYPOPUP3, _T("Dynamic Tray Icon"), _T("This icon demonstrates the dynamic icon animation support of the CTrayNotifyIcon class"), _T("Dynamic Tray Icon"), 10, CTrayNotifyIcon::BalloonStyle::Warning, &m_BitmapTrayIcon, WM_TRAYNOTIFY, IDR_TRAYPOPUP3))
    {
      ATLTRACE(_T("Failed to create tray icon 3\n"));
      return -1;
    }

    //Also create a timer used for the dynamic tray icons
    m_nTimerID = SetTimer(1, 1000, nullptr);

    return 0;
  }

  void OnFileExit(UINT /*uCode*/, int /*nID*/, HWND /*hWndCtl*/) noexcept
  {
    PostMessage(WM_CLOSE);
  }

  void OnClose()
  {
    if (m_nTimerID != 0)
    {
      KillTimer(m_nTimerID);
      m_nTimerID = 0;
    }
    SetMsgHandled(FALSE);
    PostQuitMessage(0);
  }

  LRESULT OnTrayNotification(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam)
  {
    //Delegate all the work back to the default implementation in CTrayNotifyIcon
    m_TrayIcon1.OnTrayNotification(wParam, lParam);
    m_TrayIcon2.OnTrayNotification(wParam, lParam);
    m_TrayIcon3.OnTrayNotification(wParam, lParam);

    return 0L;
  }

  void OnHappy(UINT /*uCode*/, int /*nID*/, HWND /*hWndCtl*/)
  {
    m_bHappy = !m_bHappy;

    if (m_bHappy)
    {
#pragma warning(suppress: 26446)
      m_TrayIcon1.SetIcon(m_Icons[0]);
      m_TrayIcon1.SetTooltipText(_T("I'm Happy!"));
    }
    else
    {
#pragma warning(suppress: 26446)
      m_TrayIcon1.SetIcon(m_Icons[1]);
      m_TrayIcon1.SetTooltipText(_T("I'm Sad!"));
    }
    UISetCheck(ID_HAPPY, m_bHappy);
    UISetCheck(ID_SAD, !m_bHappy);
  }

  void OnSad(UINT uCode, int nID, HWND hWndCtl)
  {
    return OnHappy(uCode, nID, hWndCtl);
  }

  void OnHide(UINT /*uCode*/, int /*nID*/, HWND /*hWndCtl*/)
  {
    m_TrayIcon1.Hide();
    UIEnable(ID_SHOW, TRUE);
  }

  void OnShow(UINT /*uCode*/, int /*nID*/, HWND /*hWndCtl*/)
  {
    m_TrayIcon1.Show();
    UIEnable(ID_SHOW, FALSE);
  }

  void OnAnimated(UINT /*uCode*/, int /*nID*/, HWND /*hWndCtl*/)
  {
    if (m_TrayIcon1.UsingAnimatedIcon())
    {
#pragma warning(suppress: 26446)
      m_TrayIcon1.SetIcon(m_Icons[0]);
      UISetCheck(ID_ANIMATED, 0);
    }
    else
    {
      m_TrayIcon1.SetIcon(m_Icons.data(), 2, 500);
      UISetCheck(ID_ANIMATED, 1);
    }
    UISetCheck(ID_HAPPY, FALSE);
    UISetCheck(ID_SAD, FALSE);
  }

  void OnAppAbout(UINT /*uCode*/, int /*nID*/, HWND /*hWndCtl*/)
  {
    if (m_pCurrentAboutDlg != nullptr)
    {
#pragma warning(suppress: 26486)
      SetForegroundWindow(m_pCurrentAboutDlg->operator HWND());
      return;
    }
  
    CAboutDlg dlg;
    m_pCurrentAboutDlg = &dlg;
    dlg.DoModal();
    m_pCurrentAboutDlg = nullptr;
  }

  void OnTimer(UINT_PTR nTimerID)
  {
    if (nTimerID == m_nTimerID)
    {
      //Draw into the tray icon bitmap
      ATL::CString sNum;
      sNum.Format(_T("%d"), m_nTimerCount);
      m_TrayIconDC.SetBkMode(TRANSPARENT);
      m_TrayIconDC.SetTextColor(RGB(255,255,255));
      const int nWidth{m_TrayIconDC.GetDeviceCaps(LOGPIXELSX)};
      const int nHeight{m_TrayIconDC.GetDeviceCaps(LOGPIXELSY)};
      CRect r;
      r.top = 0;
      r.left = 0;
      r.right = nWidth;
      r.bottom = nHeight;
      CBrush blackBrush;
      blackBrush.CreateSolidBrush(BLACK_BRUSH);
      m_TrayIconDC.FillRect(&r, blackBrush);
      m_TrayIconDC.TextOut(0, 0, sNum);

      //Update it
      m_TrayIcon3.SetIcon(&m_BitmapTrayIcon);

      //Increment the count, ready for the next time aroun
      m_nTimerCount++;
      if (m_nTimerCount == 100)
        m_nTimerCount = 0;
    }
    else
      SetMsgHandled(FALSE);
  }
};
